#!/usr/bin/php
<?php
include_once "smsacknowledge.inc.php";
include_once "/opt/monitor/op5/smsreceive/smsreceiver/utils.php";

//Load the ninja autoloader, needed for the ORM
if (!defined('SKIP_KOHANA')) {
    define('SKIP_KOHANA', true);
    require_once "/opt/monitor/op5/ninja/index.php";
}

function get_contact_by_pager($nr)
{
	$ls = op5livestatus::instance();
	list($cols, $contact, $count) = $ls->query("contacts", array("Filter: pager = $nr"), array("name"), array('auth' => false));
	if (is_array($contact) && $count >= 1 && isset($contact[0][0])) {
		return $contact[0][0];
	}
    printLog(sprintf("Couldn't find contact for pager nr: %s", $nr));

	return false;
}

function get_host_by_msg_from_service_notification($msg)
{

	$on = strstr($msg, " on ");
	if (!$on)
		return False;

	$submsg = substr($on, 4);

	$hname = preg_replace('/([^ ]*) is [A-Z].*/', '$1', $submsg);

	$ls = op5livestatus::instance();
	list($cols, $host, $count) = $ls->query("hosts", array("Filter: name = $hname"), array("name"), array('auth' => false));
	if (is_array($host) && $count >= 1 && isset($host[0][0])) {
		return $host[0][0];
	}
	printLog(sprintf("Couldn't find hostname in %s; finding message succeeded: %s; submsg: %s; hostname parsed: %s; hostname from livestatus",
        $msg, $on, $submsg, $hname));

	return '';
}

/**
 * Return sms acknowledge command string.
 *
 * @param $smsmessage array
 * @param $sticky int
 * @param $notify int
 * @param $persistent int
 * @param $hostAckString string
 * @param $serviceAckString string
 * @param $smsDelimiter string
 * @param $smsDynamicDelimiter string
 * @return string,boolean false if $smsmessage[from],$smsmessage[content] is empty
 */
function smsack($smsmessage, $sticky, $notify, $persistent, $hostAckString, $serviceAckString, $smsDelimiter, $smsDynamicDelimiter)
{
    if(!$smsmessage['from'] || !$smsmessage['content']) {
        printLog("Couldn't find contact/content from sms acknowledge message");
        return false;
    }
    $smsArr = explode($smsDelimiter, $smsmessage['content']);
	$commandString = "";

	$author = get_contact_by_pager($smsmessage['from']);
	if ($author === False) {
		printLog("No contact found with pager number $smsmessage[from]");
	} else {
		printLog("Pager number " . $smsmessage['from'] . " belongs to contact $author");
	}
	# First, look for manually witten  acknowledge messages
	if (strcasecmp($smsArr[0], $hostAckString) === 0) {
		printLog("Host ack");
		$commandString = "ACKNOWLEDGE_HOST_PROBLEM;" . $smsArr[1] . ";";
		$commandString .=  $sticky . ";" . $notify . ";" . $persistent . ";";
		$commandString .=  $author . ";" . $smsArr[2];
	} else if (strcasecmp($smsArr[0], $serviceAckString) === 0) {
		printLog("Service ack");
		$commandString = "ACKNOWLEDGE_SVC_PROBLEM;" . $smsArr[1] . ";";
		$commandString .=  $smsArr[2] . ";" . $sticky . ";" . $notify . ";" . $persistent;
		$commandString .=  ";" . $author . ";" . $smsArr[3];
	} else {
		# Fall back, try to match on sms-reply
		$tmpStr = strtok($smsmessage['content'], $smsDynamicDelimiter);
		$tmpArr = explode(" is ", $tmpStr);
		$c = count($tmpArr);
		$index = 0;
		$serviceDescr = "";
		# Try to match on service ack
		$hostName = get_host_by_msg_from_service_notification($tmpStr);
		if ($hostName != '') {
			printLog("Fallback, service ack");
			$tmpArr2 = explode(" on " . $hostName . " ", $tmpStr);
			$serviceDescr = $tmpArr2[0];
			$commandString = "ACKNOWLEDGE_SVC_PROBLEM;" . $hostName . ";";
			$commandString .=  $serviceDescr . ";" . $sticky . ";" . $notify . ";" . $persistent . ";";
			$commandString .=  $author . ";SMS acknowledgement";
		} else {
			# Final fallback, interpret message as host ack
			printLog("Fallback, host ack");
			$commandString = "ACKNOWLEDGE_HOST_PROBLEM;" . $tmpArr[0] . ";" ;
			$commandString .=  $sticky . ";" . $notify . ";" . $persistent . ";" . $author;
			$commandString .=   ";SMS acknowledgement";
		}

	}
    return $commandString;
}

/**
 * Write sms acknowledge command string to "nagios.cmd" file.
 *
 * @param $commandString string
 * @param $commandfile string
 */
function write_command_string($commandString, $commandfile) {
	printLog("About to write: '$commandString' to commandfile");
	$fHandle = fopen($commandfile, "a");
	if (!$fHandle) {
		printLog("Failed to open file '$commandfile' for writing, Aborting.");
		return;
	}
	$ret = fwrite($fHandle, "[". time() . "] " . $commandString . "\n");
	if (!$ret) {
		printLog("Failed to write file '$commandfile'");
	}
	fclose($fHandle);
}

//"Main"
if (isset($argv, $argv[0]) && realpath($argv[0]) === __FILE__) {

    if ($argc < 2) {
        printLog("Wrong number of arguments, aborting");
        exit(1);
    }

    printLog("Ack sms '$argv[1]'");
    $incommingmessage = processSmsMessage($argv[1]);

    $commandString = smsack($incommingmessage, $sticky, $notify, $persistent, $hostAckString, $serviceAckString, $smsDelimiter, $smsDynamicDelimiter);
    write_command_string($commandString, $commandfile);

    deletefile($argv[1]);

}
